import pyperclip
import os

def load_template(template_name):
    curr_path = os.path.dirname(__file__)
    file_path = os.path.join(curr_path, './' + template_name + '.py')
    template_code = ""
    with open(file_path, 'r') as f:
        template_code = f.read()

    f.close()
    pyperclip.copy(template_code)
    return template_code

def list_available_templates():
    arr = []
    arr.append('union_find')
    arr.append('trie')
    arr.append('segment_tree')
    arr.append('radix_sort')
    arr.append('preorder')
    arr.append('postorder')
    arr.append('inorder')
    print(arr)
    return arr


def bubble_sort(nums):
    
    for x in range(len(nums)):
        for y in range(len(nums)-x-1):
            if nums[y] > nums[y+1]:
                nums[y],nums[y+1] = nums[y+1],nums[y]
                
    return nums


def flatten(B):##O(B)
    nums = []
    for x in range(len(B)):
        for y in range(len(B[x])):
            nums.append(B[x][y])
            
    return nums
def radix_sort(nums,max_digits):##O(MAX_DIGTIS * (N+B))
    
    for digit in range(0,max_digits): ##O(MAX_DIGITS)
        B = [[] for x in range(10)]
        
        for item in nums:##O(N)
            thisdigit = item // 10 ** digit%10
            B[thisdigit].append(item)
            
            
        nums = flatten(B)##O(B)
        
    return nums


class TreeNode:
     def __init__(self, val=0, left=None, right=None):
        self.val = val
        self.left = left
        self.right = right

ans = []
def preorder(root):
    if root is not None:
        ans.append(root.val)
        
        if root.left:
            preorder(root.left)
            
        if root.right:
            preorder(root.right)
            



import bisect

def addRange(self, left: int, right: int) -> None:
        l = bisect.bisect_left(self.ranges, left)
        r = bisect.bisect_right(self.ranges, right)

        sub_range = []
        if l % 2 == 0: # does not overlap
            sub_range.append(left)
        if r % 2 == 0: # does not overlap
            sub_range.append(right)
        print(l,r)
        self.ranges[l:r] = sub_range
        print(self.ranges)
        
def queryRange(self, left: int, right: int) -> bool:
        l = bisect.bisect_right(self.ranges, left)
        r = bisect.bisect_left(self.ranges, right)
        return l == r and l % 2 == 1

def removeRange(self, left: int, right: int) -> None:
        l = bisect.bisect_left(self.ranges, left)
        r = bisect.bisect_right(self.ranges, right)
        sub_range = []
        if l % 2 == 1: # overlaps
            sub_range.append(left)
        if r % 2 == 1: # overlaps
            sub_range.append(right)

        self.ranges[l:r] = sub_range


from setuptools import setup, find_packages
from pathlib import Path

setup(
  name='snapalgo',
  version='0.0.7',
  description='Coding Helper',
  long_description=Path("README.md").read_text(encoding="utf-8"),
  long_description_content_type="text/markdown",
  url='https://github.com/midnightbot/snapalgo',  
  author='Anish Adnani',
  author_email='anishadnani00@gmail.com',
  license='MIT', 
  keywords=['python', 'coding', 'template', 'algo', 'algorithms'],
  packages=find_packages(),
  install_requires=['pyperclip'] ,
  classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


class TreeNode:
    def __init__(self, val=0, right=None, left=None):
        self.val = val
        self.right = right
        self.left = left

#################################### Inorder Traversal #############################

def inorder_traversal(root):
    ans = []
    def find_inorder_traversal(root):
        if root is not None:
            if root.left:
                find_inorder_traversal(root.left)
            ans.append(root.val)
            if root.right:
                find_inorder_traversal(root.right)

    find_inorder_traversal(root)
    return ans

    
#################################### Inorder Traversal #############################

#################################### Postorder Traversal #############################

def postorder_traversal(root):
    ans = []
    def find_postorder_traversal(root):
        if root is not None:
            if root.left:
                find_postorder_traversal(root.left)
            if root.right:
                find_postorder_traversal(root.right)
            ans.append(root.val)
    find_postorder_traversal(root)
    return ans

    
#################################### Postorder Traversal #############################
#################################### Preorder Traversal #############################

def preorder_traversal(root):
    ans = []
    def find_preorder_traversal(root):
        if root is not None:
            ans.append(root.val)
            if root.left:
                find_preorder_traversal(root.left)
            if root.right:
                find_preorder_traversal(root.right)

    find_preorder_traversal(root)
    return ans
#################################### Preorder Traversal #############################


## Trie Template Code
class Node:
    def __init__(self):
        self.children = {}
        self.isend = False
        
class Trie:

    def __init__(self):
        self.root = Node()
        

    def insert(self, word: str) -> None:
        ptr = self.root
        
        for x in word:
            if x not in ptr.children:
                ptr.children[x] = Node()
                
            ptr = ptr.children[x]
            
        ptr.isend = True
        

    def search(self, word: str) -> bool:
        ptr = self.root
        for x in word:
            if x not in ptr.children:
                return False
            ptr = ptr.children[x]
            
        if ptr.isend:
            return True
        else:
            return False
        

    def startsWith(self, prefix: str) -> bool:
        ptr = self.root
        for x in prefix:
            if x not in ptr.children:
                return False
            ptr = ptr.children[x]
            
        return True


#################################### Bubble sort #############################
def bubble_sort(nums):
    
    for x in range(len(nums)):
        for y in range(len(nums)-x-1):
            if nums[y] > nums[y+1]:
                nums[y],nums[y+1] = nums[y+1],nums[y]
                
    return nums

#################################### Bubble sort #############################


#################################### Insertion sort #############################
def insertion_sort(nums):
    
    for x in range(1,len(nums)): ## O(N)
        elem = nums[x]
        
        y = x-1
        
        while y>=0 and elem < nums[y]: ## O(N)
            nums[y+1] = nums[y]
            y-=1
            
        nums[y+1] = elem
        
        
    return nums

#################################### Insertion sort #############################


#################################### Merge sort #############################
def merge_sort(nums):
    
    if len(nums) > 1:
        mid = len(nums) // 2 ##split array from middle
        
        left_arr = nums[0:mid] 
        right_arr = nums[mid:] 
        
        merge_sort(left_arr)##sort left subarray
        merge_sort(right_arr)##sort right subarray
        
        i = 0
        j = 0
        k = 0
        ##merge the two sorted arrays in linear time to get a combined sorted array
        while i < len(left_arr) and j < len(right_arr):
            if left_arr[i] < right_arr[j]:
                nums[k] = left_arr[i]
                i+=1
                
            else:
                nums[k] = right_arr[j]
                j+=1
                
                
            k+=1
                
        while i < len(left_arr):
            nums[k] = left_arr[i]
            i+=1
            k+=1
            
        while j < len(right_arr):
            nums[k] = right_arr[j]
            j+=1
            k+=1
            
        return nums

#################################### Merge sort #############################

#################################### Radix sort #############################

def flatten(B):##O(B)
    nums = []
    for x in range(len(B)):
        for y in range(len(B[x])):
            nums.append(B[x][y])
            
    return nums
def radix_sort(nums,max_digits):##O(MAX_DIGTIS * (N+B))
    
    for digit in range(0,max_digits): ##O(MAX_DIGITS)
        B = [[] for x in range(10)]
        
        for item in nums:##O(N)
            thisdigit = item // 10 ** digit%10
            B[thisdigit].append(item)
            
            
        nums = flatten(B)##O(B)
        
    return nums

#################################### Radix sort #############################



## Union Find Template
parent = [-1] * 100001
        
def find_parent(x):
    if parent[x] == -1:
        return x
    parent[x] = find_parent(parent[x])
    return parent [x]

def union(x,y):
    xs = find_parent(x)
    ys = find_parent(y)